@@ -19,7 +19,16 @@
  * Visit http://goaccess.prosoftcorp.com for new releases.
  */
 
-#include <curses.h>
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#ifdef HAVE_LIBNCURSESW
+#  include <ncursesw/curses.h>
+#else
+#  include <ncurses.h>
+#endif
+
 #include <glib.h>
 #include <time.h>
 
@@ -64,6 +73,7 @@ char *ignore_host;
 int bandwidth_flag = 0;
 int host_agents_list_flag = 0;
 int http_status_code_flag = 0;
+int piping = 0;
 
 /* hash iteration */
 int iter_ctr = 0;
@@ -248,6 +258,8 @@ char *help_main[] = {
    " ^t^     [detail view] move to the first item",
    " ^b^     [detail view] move to the last item",
    "",
+   "Examples can be found by running `man goaccess`.",
+   "",
    "If you believe you have found a bug, please drop me",
    "an email with details.",
    "",
