@@ -18,7 +18,18 @@
  * Visit http://goaccess.prosoftcorp.com for new releases.
  */
 
-#include <curses.h>
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#ifdef HAVE_LIBNCURSESW
+#  include <ncursesw/menu.h>
+#  include <ncursesw/curses.h>
+#else
+#  include <ncurses.h>
+#  include <menu.h>
+#endif
+
 #include <glib.h>
 
 #ifndef COMMONS_H_INCLUDED
@@ -26,17 +37,17 @@
 
 /* Remove the __attribute__ stuff when the compiler is not GCC. */
 #if !__GNUC__
-# define __attribute__(x) /**/
+#  define __attribute__(x) /**/
 #endif
 #define BUFFER 			4096
 #define BW_HASHTABLES   3
 #define DATELEN         12
 #define DEBUG
 #define GO_UNUSED __attribute__((unused))
-#define GO_VERSION 		"0.4"
+#define GO_VERSION 		"0.4.2"
 #define MAX_CHOICES 	   300
-#define MIN_HEIGHT 		40
-#define MIN_WIDTH       97
+#define MIN_HEIGHT 		7
+#define MIN_WIDTH       0
 #define TOTAL_MODULES 	11
 /* max height of footer stdscr (rows) */
 #define MAX_HEIGHT_FOOTER 1
@@ -150,6 +161,7 @@ extern char *ignore_host;
 extern int bandwidth_flag;
 extern int host_agents_list_flag;
 extern int http_status_code_flag;
+extern int piping;
 
 /* iteration */
 extern int iter_ctr;
