@@ -25,10 +25,22 @@
 
 #include <assert.h>
 #include <ctype.h>
-#include <curses.h>
+#include <errno.h>
 #include <glib.h>
 #include <locale.h>
-#include <menu.h>
+
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#ifdef HAVE_LIBNCURSESW
+#  include <ncursesw/menu.h>
+#  include <ncursesw/curses.h>
+#else
+#  include <ncurses.h>
+#  include <menu.h>
+#endif
+
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,13 +48,13 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#include "error.h"
 #include "alloc.h"
 #include "commons.h"
-#include "util.h"
-#include "ui.h"
+#include "error.h"
 #include "parser.h"
 #include "settings.h"
+#include "ui.h"
+#include "util.h"
 
 static WINDOW *header_win, *main_win, *my_menu_win, *help_win, *schemes_win;
 
@@ -93,6 +105,7 @@ cmd_help (void)
    printf ("                 For faster parsing, don't enable this flag.\n");
    printf (" -e <argument> - Exclude an IP from being counted under the\n");
    printf ("                 HOST module. Disabled by default.\n\n");
+   printf ("Examples can be found by running `man goaccess`.\n\n");
    printf ("For more details visit: http://goaccess.prosoftcorp.com\n");
    printf ("GoAccess Copyright (C) 2009-2010 GNU GPL'd, by Gerardo Orellana");
    printf ("\n\n");
@@ -152,7 +165,8 @@ allocate_structs (int free_me)
    /* allocate 10 per module */
    MALLOC_STRUCT (s_display, 170);
 
-   /* note that the order in which we call them, is the way modules will be displayed */
+   /* note that the order in which we call them, 
+    * is the way modules will be displayed */
    generate_unique_visitors (s_display, logger);
 
    MALLOC_STRUCT (s_holder, g_hash_table_size (ht_requests));
@@ -204,8 +218,8 @@ render_screens (void)
    wattron (stdscr, COLOR_PAIR (COL_WHITE));
    generate_time ();
    chg = logger->total_process - initial_reqs;
-   mvaddstr (row - 1, 1, "[F1]Help  [O]pen detail view");
-   mvprintw (row - 1, 32, "Updated: %d - %s", chg, asctime (now_tm));
+   mvaddstr (row - 1, 1, "[F1]Help [O]pen detail view");
+   mvprintw (row - 1, 30, "%d - %s", chg, asctime (now_tm));
    mvaddstr (row - 1, col - 21, "[Q]uit Analyzer");
 
    mvprintw (row - 1, col - 5, "%s", GO_VERSION);
@@ -221,7 +235,7 @@ render_screens (void)
    /* display active label based on current module */
    wattron (header_win, COLOR_PAIR (BLUE_GREEN));
    wmove (header_win, 0, 30);
-   mvwprintw (header_win, 0, col - 20, "[Active Module %d]",
+   mvwprintw (header_win, 0, col - 19, "[Active Module %d]",
               logger->current_module);
    wattroff (header_win, COLOR_PAIR (BLUE_GREEN));
    wrefresh (header_win);
@@ -236,14 +250,15 @@ static void
 get_keys (void)
 {
    int y, x, c;
-   getmaxyx (main_win, y, x);
 
    char buf[BUFFER];
-   FILE *fp;
-   unsigned long long size1, size2;
+   FILE *fp = NULL;
+   unsigned long long size1 = 0, size2 = 0;
 
-   size1 = file_size (ifile);
+   if (!piping)
+      size1 = file_size (ifile);
    while (((c = wgetch (stdscr)) != 'q')) {
+      getmaxyx (main_win, y, x);
       switch (c) {
           /* scroll down main_win */
        case KEY_DOWN:
@@ -275,6 +290,8 @@ get_keys (void)
        case KEY_RIGHT:
        case 'o':
        case 'O':
+          if ((x < 75) || (y < 12))
+             break;             /* stdscr too small to create subwin */
           my_menu_win = create_win (main_win);
           load_popup (my_menu_win, s_holder, logger);
           touchwin (main_win);
@@ -338,21 +355,25 @@ get_keys (void)
           break;
        case 8:
        case 265:
-          help_win = newwin (y - 12, x - 40, 8, 20);
+          if ((x < 65) || (y < 12))
+             break;             /* stdscr too small to create win */
+          help_win = newwin (y - 3, 57, 5, ((x - 57) / 2));
           if (help_win == NULL)
              error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                             "Unable to allocate memory for new window.");
-          load_help_popup (help_win);
+          load_help_popup (help_win, ((x - 57) / 2));
           wrefresh (help_win);
           touchwin (main_win);
           close_win (help_win);
           break;
        case 99:
-          schemes_win = newwin (10, 40, 8, 20);
+          if ((x < 75) || (y < 12))
+             break;             /* stdscr too small to create win */
+          schemes_win = newwin (7, 57, 8, ((x - 57) / 2));
           if (schemes_win == NULL)
              error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                             "Unable to allocate memory for new window.");
-          load_schemes_win (schemes_win);
+          load_schemes_win (schemes_win, ((x - 57) / 2));
           wrefresh (schemes_win);
           touchwin (main_win);
           close_win (schemes_win);
@@ -370,13 +391,14 @@ get_keys (void)
           render_screens ();
           break;
        default:
+          if (piping)
+             break;
           size2 = file_size (ifile);
           /* file has changed */
           if (size2 != size1) {
              if (!(fp = fopen (ifile, "r")))
-                error_handler (__PRETTY_FUNCTION__,
-                               __FILE__, __LINE__,
-                               "Unable to read log file.");
+                error_handler (__PRETTY_FUNCTION__, __FILE__,
+                               __LINE__, "Unable to read log file.");
              if (!fseeko (fp, size1, SEEK_SET))
                 while (fgets (buf, BUFFER, fp) != NULL)
                    parse_log (logger, ifile, buf);
@@ -402,9 +424,6 @@ main (int argc, char *argv[])
    extern int optind, optopt, opterr;
    int row, col, o, index;
 
-   if (argc < 2)
-      cmd_help ();
-
    opterr = 0;
    while ((o = getopt (argc, argv, "f:e:sba")) != -1) {
       switch (o) {
@@ -435,7 +454,9 @@ main (int argc, char *argv[])
           abort ();
       }
    }
-   if (!ifile)
+   if (ifile != NULL && !isatty (STDIN_FILENO))
+      cmd_help ();
+   if (ifile == NULL && isatty (STDIN_FILENO))
       cmd_help ();
    for (index = optind; index < argc; index++)
       cmd_help ();
@@ -511,7 +532,7 @@ main (int argc, char *argv[])
    getmaxyx (stdscr, row, col);
    if (row < MIN_HEIGHT || col < MIN_WIDTH)
       error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
-                     "Minimum screen size - 97 columns by 40 lines");
+                     "Minimum screen size - 0 columns by 7 lines");
 
    header_win = newwin (5, col, 0, 0);
    keypad (header_win, TRUE);
