@@ -28,8 +28,18 @@
 #define _XOPEN_SOURCE 700
 
 #include <ctype.h>
-#include <curses.h>
 #include <errno.h>
+
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#ifdef HAVE_LIBNCURSESW
+#  include <ncursesw/curses.h>
+#else
+#  include <ncurses.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -575,7 +585,7 @@ process_log (struct logger *logger, char *line)
 int
 parse_log (struct logger *logger, char *filename, char *tail)
 {
-   FILE *fp;
+   FILE *fp = NULL;
    char line[BUFFER];
 
    if (tail != NULL) {
@@ -583,15 +593,25 @@ parse_log (struct logger *logger, char *filename, char *tail)
          return 1;
       return 0;
    }
-   if ((fp = fopen (filename, "r")) == NULL)
+   if (filename == NULL) {
+      fp = stdin;
+      piping = 1;
+   }
+   if (!piping && (fp = fopen (filename, "r")) == NULL)
       error_handler (__PRETTY_FUNCTION__, __FILE__, __LINE__,
                      "An error has occurred while opening the log file. Make sure it exists.");
+
    while (fgets (line, BUFFER, fp) != NULL) {
       if (process_log (logger, line)) {
-         fclose (fp);
+         if (!piping)
+            fclose (fp);
          return 1;
       }
    }
+   /* definitely not portable! */
+   if (piping)
+      stdin = freopen ("/dev/tty", "r", stdin);
+
    return 0;
 }
 
@@ -627,7 +647,7 @@ generate_unique_visitors (struct struct_display **s_display,
       else if (i == 1)
          s_display[logger->alloc_counter++]->data =
             alloc_string
-            (" HTTP requests having the same IP, same date and same agent will be considered a unique visit");
+            (" HTTP requests having the same IP, same date and same agent are considered a unique visit");
       else if (i == 2 || i == 9)
          s_display[logger->alloc_counter++]->data = alloc_string ("");
       else if (r < s_size) {
