@@ -21,6 +21,17 @@
 #ifndef UI_H_INCLUDED
 #define UI_H_INCLUDED
 
+#define T_BW         "BW"
+#define T_F_REQUESTS "Failed Requests"
+#define T_GEN_TIME   "Generation Time"
+#define T_LOG        "Log"
+#define T_REFERRER   "Referrers"
+#define T_REQUESTS   "Total Requests"
+#define T_STATIC_FIL "Static Files"
+#define T_UNIQUE404  "Unique 404"
+#define T_UNIQUE_FIL "Unique Files"
+#define T_UNIQUE_VIS "Unique Visitors"
+
 WINDOW *create_win (WINDOW * main_win);
 
 struct struct_agents
@@ -38,10 +49,10 @@ void do_scrolling (WINDOW * main_win, struct struct_display **s_display,
                    int cmd);
 void draw_header (WINDOW * win, char *header, int x, int y, int w, int color);
 void generate_time (void);
-void load_help_popup (WINDOW * help_win);
+void load_help_popup (WINDOW * help_win, size_t startx);
 void load_popup (WINDOW * my_menu_win, struct struct_holder **s_holder,
                  struct logger *logger);
-void load_schemes_win (WINDOW * schemes_win);
+void load_schemes_win (WINDOW * schemes_win, size_t startx);
 void set_input_opts (void);
 void term_size (WINDOW * main_win);
 void update_header (WINDOW * header_win, int current);
